==> japhar-0.04.tar.gz: japhar-0.04/lib/libnative/java.lang.reflect/method.c <==
/* -*- Mode: C; c-file-style: "gnu" -*-
   field.c -- native methods for java/lang/reflect/Field
   Created: Chris Toshok <toshok@hungry.com>, 12-Feb-1998.
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "jni.h"
#include "ClazzFile.h"
#include "objects.h"

JNIEXPORT jint JNICALL
Java_java_lang_reflect_Method_getModifiers(JNIEnv *env,
					   jobject method)
{
  jint slot;
  jobject clazz;
  ClazzFile *cf, *class_cf = (*env)->FindClass(env, "java/lang/Class");
  obj_thunk *o;
  jclass method_class = (*env)->FindClass(env, "java/lang/reflect/Method");
  jfieldID slot_field = (*env)->GetFieldID(env, method_class, "slot", "I");
  jfieldID clazz_field = (*env)->GetFieldID(env, method_class, "clazz", "Ljava/lang/Class;");

  slot = (*env)->GetIntField(env, method, slot_field);
  clazz = (*env)->GetObjectField(env, method, clazz_field);

  o = find_instance_data_for_class(clazz, class_cf);

  cf = (ClazzFile*)(INSTANCE_DATA(o)->fields[0].value.l);

  return cf->methods[slot]->access_flags;
}

JNIEXPORT jobject JNICALL
Java_java_lang_reflect_Method_invoke(JNIEnv *env,
				     jobject method,
				     jobject obj,
				     jobjectArray args)
{
  (*env)->FatalError(env, "java/lang/reflect/Method.invoke() not implemented yet.");
  return NULL;
}

==> japhar-0.05.tar.gz: japhar-0.05/lib/libnative/java.lang.reflect/method.c <==
/* -*- Mode: C; c-file-style: "gnu" -*-
   field.c -- native methods for java/lang/reflect/Field
   Created: Chris Toshok <toshok@hungry.com>, 12-Feb-1998.
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "jni.h"
#include "ClazzFile.h"
#include "objects.h"
#include "interp.h"

JNIEXPORT jint JNICALL
Java_java_lang_reflect_Method_getModifiers(JNIEnv *env,
					   jobject method)
{
  jint slot;
  jobject clazz;
  ClazzFile *cf, *class_cf = (*env)->FindClass(env, "java/lang/Class");
  japhar_obj o;
  jclass method_class = (*env)->FindClass(env, "java/lang/reflect/Method");
  jfieldID slot_field = (*env)->GetFieldID(env, method_class, "slot", "I");
  jfieldID clazz_field = (*env)->GetFieldID(env, method_class, "clazz", "Ljava/lang/Class;");
  FieldStruct *cf_field;
  jvalue cf_value;

  slot = (*env)->GetIntField(env, method, slot_field);
  clazz = (*env)->GetObjectField(env, method, clazz_field);

  o = cast_obj(clazz, class_cf);

  cf_field = class_cf->fields[0];

  get_instance_field(o, cf_field, &cf_value);
  cf = (ClazzFile*)cf_value.l;

  return cf->methods[slot]->access_flags;
}

JNIEXPORT jobject JNICALL
Java_java_lang_reflect_Method_invoke(JNIEnv *env,
				     jobject method,
				     jobject obj,
				     jobjectArray args)
{
  (*env)->FatalError(env, "java/lang/reflect/Method.invoke() not implemented yet.");
  return NULL;
}
